import os
import gzip
import json

import boto3


def is_gzip(file_content):
    """
    Check if the file is compressed in Gzip format.
    :param file_content:
    :return:
    """
    return file_content[:2] == b'\x1f\x8b'


def token_from_s3_bucket():
    s3 = boto3.client('s3')
    # read the file
    obj = s3.get_object(Bucket=os.environ.get('BUCKET_NAME'), Key=os.environ.get('FILE_NAME'))
    file_content = obj['Body'].read()

    # Check if the file is compressed in Gzip format
    if is_gzip(file_content):
        # If the file is compressed in Gzip format, decompress it
        file_content = gzip.decompress(file_content)

    # Decode the file content as utf-8
    decoded_content = file_content.decode('utf-8')
    token = json.loads(decoded_content.strip())
    return token


def token_to_s3_bucket(data):
    s3 = boto3.client('s3')
    # upload the file
    s3.put_object(Body=json.dumps(data), Bucket=os.environ.get('BUCKET_NAME'), Key=os.environ.get('FILE_NAME'))


def token_from_local_file():
    with open(os.environ.get('FILE_NAME'), 'r') as f:
        token = json.loads(f.read())
    return token


def token_to_local_file(data):
    with open(os.environ.get('FILE_NAME'), 'w') as f:
        f.write(json.dumps(data))



import os
from collections.abc import Mapping
import datetime as dt

import requests
from botocore.exceptions import ClientError


from storage import get_token

token_filename = 'toast_token.json'


class ToastToken(Mapping):
    """
    The ToastToken class stores the Toast API token and expiration date.
    It is a subclass of Mapping, so it can be unpacked like a dictionary.
    The token is stored in a JSON file, and is refreshed when
    the expiration date is reached.

    Using the unpacking operator:
        token = ToastToken()
        payload = {
            'locationGuid': '1234567890',
            'startDate': '2020-01-01',
            'endDate': '2020-01-02',
            **token  # Returns {"Authorization": token}
        }

    """

    def __init__(self, token_location='local'):
        if token_location == 'local':
            self.open_token_json = get_token.token_from_local_file
            self.write_token = get_token.token_to_local_file
        elif token_location == 's3':
            self.open_token_json = get_token.token_from_s3_bucket
            self.write_token = get_token.token_to_s3_bucket

        self._token, self._expiration = self.get_toast_token()
        # self._token is of the form: str - "Bearer: <token>"
        # self._expiration is a datetime object

    def to_dict(self):
        return {'Authorization': self.token}

    def __getitem__(self, key):
        return self.to_dict()[key]

    def __iter__(self):
        return iter(self.to_dict())

    def __len__(self):
        return len(self.to_dict())  # Always 1

    def write_token_json(self, token: str, expires: dt.datetime) -> None:
        data = {
            'Authorization': f"Bearer {token}",
            'Expires': expires.isoformat(),
        }
        self.write_token(data)

    def get_toast_token(self):
        try:
            token = self.open_token_json()
            expires = dt.datetime.fromisoformat(token['Expires'])
            if expires > (dt.datetime.now() + dt.timedelta(minutes=1)):
                return token['Authorization'], expires
            raise FileNotFoundError
        except (FileNotFoundError, ClientError):
            url = f'{os.environ.get("TOAST_API_SERVER")}/authentication/v1/authentication/login'

            payload = {
                "clientId": os.environ.get('TOAST_CLIENT_ID'),
                "clientSecret": os.environ.get('TOAST_CLIENT_SECRET'),
                "userAccessType": "TOAST_MACHINE_CLIENT"
            }
            headers = {"Content-Type": "application/json"}
            response = requests.post(url, json=payload, headers=headers)

            data = response.json()
            token = data['token']['accessToken']

            expires = dt.datetime.now() + dt.timedelta(seconds=data['token']['expiresIn'])
            self.write_token_json(token, expires)
            return f"Bearer {token}", expires

    @property
    def token(self):
        if self._expiration < (dt.datetime.now() - dt.timedelta(minutes=1)):
            self._token, self._expiration = self.get_toast_token()
        return self._token

    @token.setter
    def token(self, value):
        raise AttributeError("Cannot overwrite the Toast Token.")



from setuptools import setup, find_packages

setup(
    name='toast_auth',
    version='0.1.0',
    license='MIT',
    description='A package that contains a class',
    author='Turner Luke',
    author_email='turnermluke@gmail.com',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    install_requires=[
        'boto3',
        'botocore',
        'certifi',
        'charset-normalizer',
        'idna',
        'jmespath',
        'python-dateutil',
        'requests',
        's3transfer',
        'six',
        'urllib3',
    ],
)
